# cython: language_level=3
#  Drakkar-Software OctoBot-Trading
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
cimport octobot_trading.exchange_data.contracts.margin_contract as margin_contract


cdef class FutureContract(margin_contract.MarginContract):
    cdef readonly object contract_type
    cdef readonly object position_mode
    cdef public object maintenance_margin_rate

    cdef readonly double minimum_tick_size

    cdef readonly object take_profit_stop_loss_mode

    cpdef bint is_inverse_contract(self)
    cpdef bint is_perpetual_contract(self)
    cpdef bint is_one_way_position_mode(self)
    cpdef void set_position_mode(self, bint is_one_way=*, bint is_hedge=*)
    cpdef void set_take_profit_stop_loss_mode(self, object set_take_profit_stop_loss_mode)
    cpdef object get_fees_currency_side(self)
